/**
 ** Copyright (c) 2002,2009 LAAS/CNRS
 **
 ** Matthieu Herrb - Mon Apr 15 2002
 **
 **/

#include <stdio.h>
#include <math.h>
#include <portLib.h>
#include <h2mathLib.h>
#include <rFlex.h>

#include "server/rflexHeader.h"
#include "rflexConst.h"
#include "rflexClient.h"
#include "odo.h"

#ifdef RFLEX_SIMUL
#include "rflexSimul.h"
#endif

/* read and save odometer in a file */
static void rflexReadOdometer();
static void rflexSaveOdometer();

/*
 * Initialize all SDI members related to motion control
 */
STATUS
rFlexInitOdo(int *bilan)
{

  /* Default parameters for Odometry Method */
  SDI_F->odometryMode = RFLEX_ODO_2D;

  SDI_F->odo.robot.xRob = 0.;
  SDI_F->odo.robot.yRob = 0.;
  SDI_F->odo.robot.theta = 0.0;
  SDI_F->odo.robot.xRef = RFLEX_DIST_TO_CONTROLED_POINT;
  SDI_F->odo.robot.yRef = 0.;
  SDI_F->odo.robot.v = 0.;
  SDI_F->odo.robot.vt = 0.;
  SDI_F->odo.robot.w = 0.;

  /* save curvilinear abscissa */
  SDI_F->miscOdo.s = 0;
  rflexReadOdometer();

  /* vxx */
  SDI_F->odo.odoError.var[0] = SIGMA3_TO_VARIANCE(RFLEX_INIT_ERR_X);
  /* vxy */
  SDI_F->odo.odoError.var[1] = 0;
  /* vyy */
  SDI_F->odo.odoError.var[2] = SIGMA3_TO_VARIANCE(RFLEX_INIT_ERR_Y);
  /* vxt */
  SDI_F->odo.odoError.var[3] = 0;
  /* vyt */
  SDI_F->odo.odoError.var[4] = 0;
  /* vtt */
  SDI_F->odo.odoError.var[5] = SIGMA3_TO_VARIANCE(RFLEX_INIT_ERR_THETA);

  SDI_F->speedRef.v = 0.0;
  SDI_F->speedRef.w = 0.0;
  SDI_F->speedRef.vmax = RFLEX_VMAX_ROBOT;
  SDI_F->speedRef.wmax = RFLEX_WMAX_ROBOT;
  SDI_F->speedRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
  SDI_F->speedRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;

  SDI_F->trackStr.trackMode = GENPOS_NO_TRACKING;
  SDI_F->id = 0;

  SDI_F->varParams.coeffLinAng = RFLEX_INIT_VAR_COEFF_LIN_ANG;
  SDI_F->varParams.coeffLin = RFLEX_INIT_VAR_COEFF_LIN;
  SDI_F->varParams.coeffAng = RFLEX_INIT_VAR_COEFF_ANG;

  /* Parametres cinematiques boucle d'asservissement */
  SDI_F->cmd.distPoint = RFLEX_DIST_TO_CONTROLED_POINT;
  SDI_F->cmd.KpLongit = RFLEX_KP_LONGIT;
  SDI_F->cmd.KiLongit = RFLEX_KI_LONGIT;
  SDI_F->cmd.KpAng    = RFLEX_KP_ANG;
  SDI_F->cmd.KiAng    = RFLEX_KI_ANG;
  SDI_F->cmd.KpTransv = RFLEX_KP_TRANSV;
  SDI_F->cmd.KiTransv = RFLEX_KI_TRANSV;
  SDI_F->cmd.longitErrorMax = RFLEX_LONGIT_ERROR_MAX;
  SDI_F->cmd.longitMaxErrSum = RFLEX_LONGIT_MAX_ERR_SUM;
  SDI_F->cmd.transvErrorMax = RFLEX_TRANSV_ERROR_MAX;
  SDI_F->cmd.transvMaxErrSum = RFLEX_TRANSV_MAX_ERR_SUM;
  SDI_F->cmd.angErrorMax = RFLEX_ANG_ERROR_MAX;
  SDI_F->cmd.angMaxErrSum = RFLEX_ANG_MAX_ERR_SUM;

 /* Initialiser les valeurs de consignes */
  SDI_F->configRef.backFlag = GENPOS_FORWARD_MOTION;
  SDI_F->configRef.numRef = 0;
  SDI_F->configRef.x = SDI_F->cmd.distPoint;
  SDI_F->configRef.y = 0.;
  SDI_F->configRef.theta = 0.;
  SDI_F->configRef.v = 0.;
  SDI_F->configRef.w = 0.;
  SDI_F->configRef.vmax = RFLEX_VMAX_ROBOT;
  SDI_F->configRef.wmax = RFLEX_WMAX_ROBOT;
  SDI_F->configRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
  SDI_F->configRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;

  return OK;
}

/*
 * Send commands to control the wheels
 */
STATUS
Asserv(struct reflex *reflex, int *bilan)
{
	static int myTick = 0, myTick2 = 0;
	int trq, acc, vel;
	unsigned char dir;
	BOOL cmdSent = FALSE;


	if (SDI_F->activeReq & RFLEX_MOT_SET_DIR) {
		/* wait for previous command to complete */
		return OK;
	}
	/* Translation axis */
	if (1 /*SDI_F->speedRef.v != 0.0*/) {
	  /* Send real request or simul */
#if ! defined RFLEX_SIMUL
		vel = M_TO_STEP(SDI_F->speedRef.v);
		acc = M_TO_STEP(SDI_F->speedRef.linAccelMax);
		dir = (vel<0 ? 0 : 1);
		vel = abs(vel);
		trq = RFLEX_LIN_TRQ_DEFAULT;
		SDI_F->activeReq |= RFLEX_MOT_SET_DIR;
		reflex->motion->setDirReq(reflex, SDI_F->id, 0,
					  vel,  acc,  trq, dir);
#else
		/* Simulate motion */
		*bilan = rflexSimul(SDI_F->speedRef.v, 0);
#endif
		cmdSent = TRUE;
	}
	/* Rotation axis */
	if (1/*SDI_F->speedRef.w != 0.0*/) {
		/* Send real request or simul */
#if ! defined RFLEX_SIMUL
		vel = RAD_TO_STEP(SDI_F->speedRef.w);
		acc = RAD_TO_STEP(SDI_F->speedRef.angAccelMax);
		dir = (vel<0 ? 0 : 1);
		vel = abs(vel);
		trq = RFLEX_ANG_TRQ_DEFAULT;
		SDI_F->activeReq |= RFLEX_MOT_SET_DIR;
		reflex->motion->setDirReq(reflex, SDI_F->id, 1,
					  vel,  acc,  trq, dir);
#else
		/* Simulate motion */
		*bilan = rflexSimul(SDI_F->speedRef.w, 1);
#endif
		cmdSent = TRUE;
	}
	if (cmdSent) {
		/* Increment the command identifier */
		INCREMENT(SDI_F->id);
	}
	/* Periodically update brakes and battery status */
	if (myTick++ == 10) {
		reflex->system->statusReq(reflex, SDI_F->id);
		INCREMENT(SDI_F->id);
		myTick = 0;
	}
	/* Test joystick state to detect switched to manual mode */
	if (myTick == 5 && SDI_F->systemState.mode != MANUAL_MODE) {
		reflex->joystick->stateReq(reflex, SDI_F->id);
		INCREMENT(SDI_F->id);
		SDI_F->activeReq |= RFLEX_JOY_GET_STATE;
	}
	/* Periodically save odometer */
	if (myTick2++ == 1000) {
	        rflexSaveOdometer();
		myTick2 = 0;
	}

#ifdef RFLEX_SIMUL
	SDI_F->activeReq &= ~RFLEX_MOT_SET_DIR;
#endif

	return OK;
}

/*
 * Compute commands from joystick
 */
STATUS
JoystickDrive(struct reflex *reflex, int *bilan)
{
	double v, w;
#if 0
	static int myTick = 0;
#endif

	if (SDI_F->activeReq & RFLEX_JOY_GET_STATE) {
		/* wait for joystick status update */
		return OK;
	}
	if (SDI_F->joystickState.buttons == RFLEX_JOYSTICK_BUTTON1) {
#if 0
		/* debug */
		if (myTick++ == 25) {
			printf("x y %d %d\n", SDI_F->joystickState.x,
			       SDI_F->joystickState.y);
			myTick = 0;
		}
#endif
		/* Linear speed */
		v = SDI_F->joystickState.y * 0.005;
		if (fabs(v) > RFLEX_VMAX_ROBOT/2.0) {
			/* limit speed */
			v = SIGN(v)*RFLEX_VMAX_ROBOT/2.0;
		}
		SDI_F->speedRef.v = v;
		SDI_F->speedRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
		/* angular speed */
		w = - SDI_F->joystickState.x * 0.0035;
		if (fabs(w) > RFLEX_WMAX_ROBOT/2.0) {
			/* limit speed */
			w = SIGN(w)*RFLEX_WMAX_ROBOT/2.0;
		}
		SDI_F->speedRef.w = w;
		SDI_F->speedRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;
	} else {
		SDI_F->speedRef.v = 0.0;
		SDI_F->speedRef.w = 0.0;
		SDI_F->speedRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
		SDI_F->speedRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;
	}

	/* Request a Joystick update */
	reflex->joystick->stateReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);
	SDI_F->activeReq |= RFLEX_JOY_GET_STATE;

	return OK;
}


/* ----------------------------------------------------------------------
 *
 *  rflexReadOdometer  -  reads cumulated motion odometer from a file
 *
 *
 *
 */

static const char* rflexFileName = RFLEX_ODO_LOG_NAME;
static FILE *rflexFile;

static void rflexReadOdometer()
{
    if (! rflexFileName)
        return;

  char buf[256];
  long int sInit=-1, sInit_tmp;
  long int date=-1, date_tmp;

  if (!(rflexFile = fopen(rflexFileName, "r"))) {
    printf ("rflex: cannot open odometer file %s\n", rflexFileName);
    rflexFile = 0;
    return;
  }

  while(fgets(buf, 256, rflexFile) &&
	sscanf(buf, "%ld %ld", &date_tmp, &sInit_tmp) == 2) {
    date = date_tmp;
    sInit = sInit_tmp;
  }
  fclose (rflexFile);

  if (sInit == -1) {
    printf ("rflex: cannot read init odometer in %s\n", rflexFileName);
    rflexFile = NULL;
    SDI_F->miscOdo.sInit = 0;
    return;
  }

  printf ("rflex: init odometer = %ld m\n", sInit);
  SDI_F->miscOdo.sInit = sInit;
}

/* ----------------------------------------------------------------------
 *
 *  rflexSaveOdometer  -  saves cumulated motion odometer in a file
 *
 *
 *
 */

#include <sys/time.h>

static void rflexSaveOdometer()
{
  struct timeval tv;
  static int prevS;

  /* test if store dist file was found */
  if (! rflexFileName)
    return;
  if (!rflexFile)
    return;

  /* need at least 1 meter motion */
  if (prevS+1 > SDI_F->miscOdo.s)
    return;
  prevS = (int)SDI_F->miscOdo.s;

  /* get date */
  gettimeofday(&tv, NULL);

  /* open file */
  if (!(rflexFile = fopen(rflexFileName, "a"))) {
    printf ("rflex: cannot open file %s\n", rflexFileName);
    rflexFile = 0;
    return;
  }

  /* save to file */
  if (fprintf(rflexFile, "%ld  %ld\n",
	      tv.tv_sec, (long int)(SDI_F->miscOdo.s + SDI_F->miscOdo.sInit)) == 0) {
    printf ("rflex: cannot save odometer in %s\n", rflexFileName);
    rflexFile = 0;
    return;
  }
  printf ("rflex : save odometer %ld in %s\n", SDI_F->miscOdo.sInit, rflexFileName);

  fclose (rflexFile);
}

