/**
 ** rflexCntrlTaskCodels.c
 **
 ** Codels used by the control task rflexCntrlTask
 **
 ** Auteur:
 ** Date:
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <math.h>
#include <stdio.h>
#include "server/rflexHeader.h"

#include "rflexConst.h"
#include "rflexCodels.h"

#ifdef RFLEX_SIMUL
#include <rFlex.h>
#include "rflexSimul.h"
#endif

/*------------------------------------------------------------------------
 * SetMode  -  control codel of CONTROL request SetMode
 *
 * Description:
 * Report (bilan): OK
 *
 * Return:    OK ou ERROR
 */

STATUS
SetMode(RFLEX_MODE *mode, int *bilan)
{
	if ((SDI_F->trackStr.trackMode == GENPOS_TRACK_SPEED) ||
	    (SDI_F->trackStr.trackMode == GENPOS_TRACK_CONFIG)) {
		*bilan = S_rflex_TRACKING_ON;
		return ERROR;
	}
	return OK;
}

/*------------------------------------------------------------------------
 * SetWdogRef  -  control codel of CONTROL request SetWdogRef
 *
 * Description:    verifies if WDOG is coherent
 * Report (bilan): OK
 *
 * Return:    OK ou ERROR
 */

STATUS
SetWdogRef(unsigned int *WatchDogRef, int *bilan)
{
  return OK;
}



/*------------------------------------------------------------------------
 * Recale  -  control codel of CONTROL request SetPos
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflex_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS
Recale (RFLEX_CART_3DCONFIG *setPosStr, int *bilan)
{
#ifdef RFLEX_SIMUL
	/* tell the simulation function that position has changed */
	rflexSimulInit();
#endif
	SDI_F->odo.robot.xRob = setPosStr->x;
	SDI_F->odo.robot.yRob = setPosStr->y;
	SDI_F->odo.robot.zRob = setPosStr->z;
	SDI_F->odo.robot.theta = setPosStr->theta;

	SDI_F->gyro.gyroToRobotOffset = SDI_F->odo.robot.theta
					- SDI_F->gyro.gyroTheta;

	return OK;
}

/*------------------------------------------------------------------------
 * controlPosErr  -  control codel of CONTROL request SetPosErr
 *
 * Description:
 * Report (bilan): OK
 *
 * Return:    OK ou ERROR
 */

STATUS
controlPosErr (GENPOS_CART_CONFIG_VAR *setVarTab, int *bilan)
{

	return OK;
}

/*------------------------------------------------------------------------
 * VarParamsCntrl  -  control codel of CONTROL request SetVarParams
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflex_BAD_CMD_DATA
 *
 * Return:    OK ou ERROR
 */

STATUS
VarParamsCntrl (RFLEX_VAR_PARAMS *varParams, int *bilan)
{
	return OK;
}

/*------------------------------------------------------------------------
 * arretEnAvant  -  control codel of CONTROL request Stop
 *
 * Description:
 * Report (bilan): OK
 *
 * Return:    OK ou ERROR
 */

STATUS
arretEnAvant (int *bilan)
{
	SDI_F->speedRef.v = 0.0;
	SDI_F->speedRef.w = 0.0;
	return OK;
}

/*------------------------------------------------------------------------
 * GotoSpeed  -  control codel of CONTROL request GotoSpeed
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflex_BAD_REF_DATA
 *                 S_rflex_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS
GotoSpeed (GENPOS_CART_SPEED *speedRef, int *bilan)
{
	if (!rflexReadyForMotion(bilan))
		return ERROR;
	if (fabs(speedRef->v) > RFLEX_VMAX_ROBOT ||
	    fabs(speedRef->w) > RFLEX_WMAX_ROBOT) {
		*bilan = S_rflex_BAD_REF_DATA;
		printf ("rflex : BAD SPEED REF\n");
		return ERROR;
	}
	if (fabs(speedRef->linAccelMax) > RFLEX_AMAX_LIN_ROBOT ||
	    fabs(speedRef->angAccelMax) > RFLEX_AMAX_ANG_ROBOT) {
		*bilan = S_rflex_BAD_REF_DATA;
		printf ("rflex : BAD ACCEL REF\n");
		return ERROR;
	}
	return OK;
}


/*------------------------------------------------------------------------
 * TrackSpeedCntrl  -  control codel of EXEC request TrackSpeedStart
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflexPumpTask_POSTER_NOT_FOUND
 *                 S_rflexPumpTask_BAD_REF
 *                 S_rflexPumpTask_CMD_STOP_TRACK
 *                 S_rflexPumpTask_POSTER_NOT_UPDATED
 *
 * Return:    OK ou ERROR
 */

STATUS
TrackSpeedCntrl(GENPOS_POSTER_NAME *posterName, int *bilan)
{
	if (!rflexReadyForMotion(bilan))
		return ERROR;
	/* Switch to traking mode */
	SDI_F->trackStr.trackMode = GENPOS_TRACK_SPEED;
	return OK;
}



/*------------------------------------------------------------------------
 * rflexTrackConfigCntrl  -  control codel of EXEC request TrackConfigStart
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflexPumpTask_POSTER_NOT_FOUND
 *                 S_rflexPumpTask_BAD_REF
 *                 S_rflexPumpTask_CMD_STOP_TRACK
 *                 S_rflexPumpTask_POSTER_NOT_UPDATED
 *
 * Return:    OK ou ERROR
 */

STATUS
rflexTrackConfigCntrl(GENPOS_POSTER_NAME *posterName, int *bilan)
{
	if (!rflexReadyForMotion(bilan))
		return ERROR;
	/* Switch to traking mode */
	SDI_F->trackStr.trackMode = GENPOS_TRACK_CONFIG;
	return OK;
}

/*------------------------------------------------------------------------
 * SetGeomConfigCntrl  -  control codel of CONTROL request SetGeoConfig
 *
 * Description:
 * Report (bilan): OK
 *                 S_rflex_BAD_CMD_DATA
 *                 S_rflex_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS
SetGeomConfigCntrl(RFLEX_GEOM_PARAM_STR *geomParams, int *bilan)
{
	/* What can be tested on the validity of the parameters ? */
	return OK;
}


