/**
 ** Copyright (c) 2004,2009 LAAS/CNRS
 **
 ** Sara Fleury
 **
 **/

#include <math.h>
#include <stdio.h>

#include "rflexConst.h"
#include "server/rflexHeader.h"

#include "rflexCodels.h"


/*------------------------------------------------------------------------
 * rflexStopTracking
 *
 */

void rflexStopTracking(void)
{
  /* Stop tracking */
  SDI_F->speedRef.numRef = 0;
  SDI_F->trackStr.trackMode = GENPOS_NO_TRACKING;

  SDI_F->speedRef.v = 0.0;
  SDI_F->speedRef.w = 0.0;
  SDI_F->speedRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
  SDI_F->speedRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;

  SDI_F->configRef.x = 0.;
  SDI_F->configRef.y = 0.;
  SDI_F->configRef.theta = 0.;
  SDI_F->configRef.v = 0.;
  SDI_F->configRef.w = 0.;
  SDI_F->configRef.linAccelMax = RFLEX_AMAX_LIN_ROBOT;
  SDI_F->configRef.angAccelMax = RFLEX_AMAX_ANG_ROBOT;
  SDI_F->configRef.numRef = 0;

}

/*------------------------------------------------------------------------
 * rflexReadyForMotion
 *
 * Return:    TRUE or FALSE
 */

BOOL
rflexReadyForMotion(int *bilan)
{
  static RFLEX_SYSTEM_STATE_STR prevSystemState;
  static RFLEX_MODE prevMode;

  /* print state change */
  if (rflexDataStrId->systemState.shock != prevSystemState.shock) {
    if (rflexDataStrId->systemState.shock)
      printf ("rflex : SHOCK !\n");
    else
      printf ("rflex : SHOCK off\n");
    prevSystemState.shock = rflexDataStrId->systemState.shock;
  }

  if (rflexDataStrId->systemState.brake != prevSystemState.brake) {
    if (rflexDataStrId->systemState.brake)
      printf ("rflex : BRAKES ON !\n");
    else
      printf ("rflex : BRAKES off\n");
    prevSystemState.brake = rflexDataStrId->systemState.brake;
  }

  if (rflexDataStrId->systemState.batteryStatus != prevSystemState.batteryStatus) {
    printf ("rflex : BATTERY LEVEL = %d / 2 (%f V) \n",
	    rflexDataStrId->systemState.batteryStatus, rflexDataStrId->systemState.batteryLevel);
    prevSystemState.batteryStatus = rflexDataStrId->systemState.batteryStatus;
  }

  if (rflexDataStrId->systemState.mode != prevMode) {
    if (rflexDataStrId->systemState.mode == MANUAL_MODE)
      printf ("rflex : MANUAL_MODE\n");
    else
      printf ("rflex : AUTO_MODE\n");
    prevMode = rflexDataStrId->systemState.mode;
  }

  /* record current state */
  if (rflexDataStrId->systemState.shock) {
    *bilan = S_rflex_SHOCK;
    return FALSE;
  }

  if (rflexDataStrId->systemState.brake) {
    *bilan = S_rflex_BRAKES_ON;
    return FALSE;
  }

  if (rflexDataStrId->systemState.mode == MANUAL_MODE) {
    *bilan = S_rflex_MANUAL;
    return FALSE;
  }
  return TRUE;
}


/*------------------------------------------------------------------------
 * rflexGetReport
 *
 */

int rflexGetStatusReport(void)
{
  if (rflexDataStrId->systemState.shock)
    return S_rflex_SHOCK;

  if (rflexDataStrId->systemState.mode == MANUAL_MODE)
    return S_rflex_MANUAL;

  if (rflexDataStrId->systemState.brake)
    return S_rflex_BRAKES_ON;

  return OK;
}
