/**
 ** rflexPosterTaskCodels.c
 **
 ** Codels appeles par la tache rflexPosterTask
 **
 ** Auteur:
 ** Date:
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <stdio.h>
#include <math.h>

#include "h2mathLib.h"
#include "server/rflexHeader.h"
#include "rflexConst.h"
#include "rflexSonars.h"
#include "pomStruct.h"		/* pom ME client library */
#include "libeuler/pomEuler.h"

/* --- Global variables ---------------------------------------------- */

/* pom */

POSTER_ID	       	pomFrameId=NULL;	/* lloco frame */
POM_SENSOR_POS		firstFrame;	/* first pom pos */
POM_SENSOR_POS		frame;		/* pom pos */

static POM_EULER_V		sensorPos;
static POM_EULER_V              tframe;


/*
 * --- rflexPosterInit --------------------------------------------------
 *
 * Codel d'initialisation (SDI/f, ...)
 */


STATUS
rflexPosterTaskInit(int *bilan)
{
   RFLEX_SONARSCONFIG_POSTER_STR sonarConfig = {{RFLEX_SONARS_CONFIG_CONST}};

   pomFrameId = NULL; /* unitialized */
   SDI_F->pomTagging = RFLEX_DEFAULT_TAGGING;

   SDI_F->sonarsConfig = sonarConfig.Config;

   if (rflexSonarsConfigPosterWrite(RFLEX_SONARSCONFIG_POSTER_ID,
				   &sonarConfig) == ERROR) {
     printf ("rflexPosterTaskInit: failed rflexSonarConfigPosterWrite\n");
     return ERROR;
   }

   return OK;
}


/*
 * --- rflexPosterEnd ---------------------------------------------------
 *
 */

STATUS
rflexPosterTaskEnd()
{
   return OK;
}


/*------------------------------------------------------------------------
 *
 * PosterTrackPerm  --  codel de l'activite' permanente
 *
 * Description:
 *
 * Bilans:      OK
 *
 * Retourne:    OK ou ERROR
 */

STATUS
PosterTrackPerm(int *bilan)
{

   /* Get the pom's current position */
   if (pomFrameId == NULL && SDI_F->pomTagging == RFLEX_ENABLE_POMTAGS) {
      if (posterFind("pomRobotFramePos",&pomFrameId) != OK) {
	fputs("rflex: could not get frame\n", stderr);
	pomFrameId = NULL;
	*bilan = S_rflex_POSTER_NOT_FOUND;
	return OK;
      }
      if (rflexPOM_SENSOR_POSPosterRead(pomFrameId,&firstFrame) != OK) {
	fputs("rflex: could not read frame\n", stderr);
	pomFrameId = NULL;
	*bilan = S_rflex_POSTER_NOT_FOUND;
	return OK;
      }
   }

   /* fill a POM_EULER_V with current odo position */
   sensorPos.euler.yaw   = SDI_F->odo.robot.theta;
   sensorPos.euler.pitch = SDI_F->odo.robot.pitch;
   sensorPos.euler.roll  = SDI_F->odo.robot.roll;
   sensorPos.euler.x = SDI_F->odo.robot.xRob;
   sensorPos.euler.y = SDI_F->odo.robot.yRob;
   sensorPos.euler.z = SDI_F->odo.robot.zRob;

#if 1 /* should be done but for now we are tracking a bug */
   memset((void *)&sensorPos.var, 0, sizeof(sensorPos.var));
   sensorPos.var.voo = SDI_F->odo.odoError.var[5]; /* tt */
   sensorPos.var.vox = SDI_F->odo.odoError.var[3]; /* xt */
   sensorPos.var.vxx = SDI_F->odo.odoError.var[0]; /* xx */
   sensorPos.var.voy = SDI_F->odo.odoError.var[4]; /* yt */
   sensorPos.var.vxy = SDI_F->odo.odoError.var[1]; /* xy */
   sensorPos.var.vyy = SDI_F->odo.odoError.var[2]; /* vyy */
#endif

   /* compute new position */
   if (pomFrameId != NULL && SDI_F->pomTagging == RFLEX_ENABLE_POMTAGS) {
     if (rflexPOM_SENSOR_POSPosterRead(pomFrameId,&frame) != OK) {
	 fputs("rflex: could not read frame\n", stderr);
	 pomFrameId = NULL;
	 *bilan = S_rflex_POSTER_NOT_FOUND;
	 return OK;
      }

     /* We deduce the estimated position of the robot in the base */
     tframe = frame.sensorToMain;
     pomEulerVCompIn(&SDI_F->posPoster.main,
		     pomEulerVInvert(&tframe), &sensorPos);

     /* But don't forget to recompose by the sensor/main frame
	(the one of the beginning to get the ROBOT FRAME displacement */
     tframe = firstFrame.sensorToMain;
     pomEulerVComp(&SDI_F->posPoster.main,&firstFrame.sensorToMain);

     SDI_F->posPoster.date1 = frame.date;
   }

   /* pom not available */
   else {
     SDI_F->posPoster.date1 = 0;
     SDI_F->posPoster.main = sensorPos;
   }

   /* On s'occupe maintenant des vitesse */
   SDI_F->posPoster.VLocal.euler.yaw = SDI_F->odo.robot.w;
   SDI_F->posPoster.VLocal.euler.x = SDI_F->odo.robot.v;

   /* update poster */
   SDI_F->posPoster.kind = POM_ME_DELTA;
   SDI_F->posPoster.confidence = 0.2; /* xxx */

/*    if (!count) { */
/*      count = 20; */
/*      printf ("theta %5.3f\n", RAD_TO_DEG(SDI_F->odo.robot.theta)); */
/*    } */
/*    else  */
/*      count--; */

   /* Exports sonar sensors coordinates */
   if (rFlexComputeSensorsEchoes(bilan) == ERROR) {
     printf("rflexPosterTrackPerm: erreur dans le calcul des echos us\n");
     *bilan = S_rflex_POSTER_NOT_FOUND;
     return OK;
   }

   return OK;
}

