/**
 ** rflexPumpTaskCodels.c
 **
 ** Codels appeles par la tache rflexPumpTask
 **
 ** Auteur:
 ** Date:
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <math.h>
#include <stdio.h>
#include <string.h>

#include "rflexConst.h"
#include "server/rflexHeader.h"
#include "rflexCodels.h"

/* library of control laws */
#ifdef USE_CONTROL_LAWS
#include "contLawProto.h"
#endif


/*-------- Les variables statiques ---------*/
static double prevV = 0.;      /* Vitesse linaire a la periode precedente */
static double prevW = 0.;      /* Vitesse angulaire periode precedente */


 /*-------------*/
 /* TRACK START */
 /*-------------*/

/*------------------------------------------------------------------------
 * TrackSpeedStart
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_POSTER_NOT_FOUND
 *              S_rflex_BAD_REF
 *              S_rflex_CMD_STOP_TRACK
 *              S_rflex_POSTER_NOT_UPDATED
 */

/* TrackSpeedStart  -  codel START de TrackSpeedStart
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
TrackSpeedStart (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  POSTER_ID posterPumpedId;
  GENPOS_CART_SPEED order;

  /* Find the poster identifier */
  if (posterFind((char *)posterName, &posterPumpedId) != OK) {
    *bilan = S_rflex_POSTER_NOT_FOUND;
    return ETHER;
  }
  SDI_F->trackStr.posterId = (int)posterPumpedId;

  /* First read */
  if (posterRead(posterPumpedId, 0,
		 &order, sizeof(GENPOS_CART_SPEED))
      != sizeof(GENPOS_CART_SPEED)) {
    *bilan = S_rflex_POSTER_NOT_FOUND;
    return ETHER;
  }
  SDI_F->speedRef.numRef = 1;

  return EXEC;
}


/* rflexTrackConfigStart   -  codel START de TrackConfigStart
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
#ifdef USE_CONTLAWS
ACTIVITY_EVENT
rflexTrackConfigStart (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  POSTER_ID posterPumpedId;
  GENPOS_CART_REF order;

  /* Find the poster identifier */
  if (posterFind((char *)posterName, &posterPumpedId) != OK) {
    *bilan = S_rflex_POSTER_NOT_FOUND;
    return ETHER;
  }
  SDI_F->trackStr.posterId = (int)posterPumpedId;

  /* First read */
  if (posterRead(posterPumpedId, 0,
		 &order, sizeof(GENPOS_CART_REF))
      != sizeof(GENPOS_CART_REF)) {
    *bilan = S_rflex_POSTER_NOT_FOUND;
    return ETHER;
  }
  SDI_F->speedRef.numRef = 1;

  return EXEC;
}
#else
ACTIVITY_EVENT
rflexTrackConfigStart (GENPOS_POSTER_NAME *posterName, int *bilan)
{ return ETHER; }
#endif


/*------------*/
/* TRACK EXEC */
/*------------*/

/* TrackSpeedExec  -  codel EXEC de TrackSpeedStart
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
TrackSpeedExec (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  static int noChange = 0;
  static int prevNumRef = 0;
  int prevRflexNumRef;

  GENPOS_CART_SPEED order;

  /* Internal interrupt */
  if (SDI_F->trackStr.trackMode != GENPOS_TRACK_SPEED) {
    if ((*bilan = rflexGetStatusReport()) == OK)
      *bilan = S_rflex_CMD_STOP_TRACK;
    SDI_F->speedRef.numRef = 0;
    return END;
  }
  /* Read poster */
  if (posterRead((POSTER_ID)SDI_F->trackStr.posterId, 0,
		 &order, sizeof(GENPOS_CART_SPEED))
      != sizeof(GENPOS_CART_SPEED)) {
    *bilan = errnoGet();
    return END;
  }
  /* Check that reference is updated, if speeds are not nul */
  if (order.numRef == prevNumRef) {
    if (fabs(order.v) > 0.001 ||
	fabs(order.w) > 0.001) {
      noChange++;
      if (noChange > SDI_F->WatchDogRef) {
	*bilan = S_rflex_POSTER_NOT_UPDATED;
	printf("rflexTrackSpeedExec: no change "
	       "for %d times!\n", noChange);
	noChange = 0;
	return END;
      }
    } else {
      noChange = 0;
    }
  } else {
    prevNumRef = order.numRef;
    noChange = 0;

    /* Check that orders are in acceptable range */
    if (fabs(order.v) > RFLEX_VMAX_ROBOT ||
	fabs(order.w) > RFLEX_WMAX_ROBOT ||
	fabs(order.vmax) > RFLEX_VMAX_ROBOT ||
	fabs(order.wmax) > RFLEX_WMAX_ROBOT ||
	order.linAccelMax <= 0. ||
	order.linAccelMax > RFLEX_AMAX_LIN_ROBOT ||
	order.angAccelMax <= 0. ||
	order.angAccelMax > RFLEX_AMAX_ANG_ROBOT) {
      *bilan = S_rflex_BAD_REF;
      printf("bad ref: v=%f w=%f linAccMax=%f angAccMax=%f\n", order.v, order.w,
	     order.linAccelMax, order.angAccelMax);
      return END;
    }
  }
  /* Send the command */
  prevRflexNumRef = SDI_F->speedRef.numRef;
  memcpy(&(SDI_F->speedRef), &order, sizeof(GENPOS_CART_SPEED));
  /* 	printf("ref: %f %f %f %f\n", order.v, order.w, */
  /* 	       order.linAccelMax, order.angAccelMax); */
  /* Signal the reference update */
  SDI_F->speedRef.numRef = prevRflexNumRef == 0 ? 1 : 0;

  return EXEC;
}


/* rflexTrackConfigExec  -  codel EXEC de TrackConfigExec
   Retourne:  EXEC END ETHER FAIL ZOMBIE
   Compute the command law and write it in the SDI_F      */
#ifdef USE_CONTLAWS
ACTIVITY_EVENT
rflexTrackConfigExec (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  static BOOL rflexImportantDriftFlag=FALSE;
  static int noChange = 0;
  static int prevNumRef = 0;
  int prevRflexNumRef;
  GENPOS_CART_REF order;

  /* Internal interrupt */
  if (SDI_F->trackStr.trackMode != GENPOS_TRACK_CONFIG) {
    if ((*bilan = rflexGetStatusReport()) == OK)
      *bilan = S_rflex_CMD_STOP_TRACK;
    SDI_F->speedRef.numRef = 0;
    SDI_F->configRef.numRef=0;
    return END;
  }
  /* Read poster */
  if (posterRead((POSTER_ID)SDI_F->trackStr.posterId, 0,
		 &order, sizeof(GENPOS_CART_REF))
      != sizeof(GENPOS_CART_REF)) {
    *bilan = errnoGet();
    return END;
  }
  /* Check that reference is updated, if speeds are not nul */
  if (order.numRef == prevNumRef) {
    if (fabs(order.v) > 0.001 ||
	fabs(order.w) > 0.001) {
      noChange++;
      if (noChange > SDI_F->WatchDogRef) {
	*bilan = S_rflex_POSTER_NOT_UPDATED;
	printf("rflexTrackConfigExec: no change "
	       "for %d times!\n", noChange);
	noChange = 0;
	return END;
      }
    } else {
      noChange = 0;
    }
  } else {
    prevNumRef = order.numRef;
    noChange = 0;

    /* Check that orders are in acceptable range */
    if (fabs(order.v) > RFLEX_VMAX_ROBOT ||
	fabs(order.w) > RFLEX_WMAX_ROBOT ||
	order.linAccelMax <= 0. ||
	order.linAccelMax > RFLEX_AMAX_LIN_ROBOT ||
	order.angAccelMax <= 0. ||
	order.angAccelMax > RFLEX_AMAX_ANG_ROBOT) {
      *bilan = S_rflex_BAD_REF;
      printf("rflex bad ref: v=%f w=%f linAccMax=%f angAccMax%f\n", order.v, order.w,
	     order.linAccelMax, order.angAccelMax);
      return END;
    }
  }


  /* Compute the command */
  /* We should use the return value DriftFlag...*/
  rflexImportantDriftFlag =
    contLawCartConfigControl ( &(rflexDataStrId->cmd),
			       &order,
			       &(rflexDataStrId->odo.robot),
			       EXEC_TASK_PERIOD(RFLEX_PUMPTASK_NUM),
			       prevV,  prevW,
			       &(order.v), &(order.w));

  /* Store values */
  prevRflexNumRef = SDI_F->configRef.numRef;
  prevV = order.v;
  prevW = order.w;

  /* Update SDI */
  /* we must update both speedRef and ref, because the robot
     tracks on ref but moves due to speedRef update */

  /** Copy ref **/
  memcpy(&(SDI_F->configRef), &order, sizeof(GENPOS_CART_REF));
  /* Signal the reference update */
  SDI_F->configRef.numRef = prevRflexNumRef == 0 ? 1 : 0;

  /** Copy speedRef **/
  SDI_F->speedRef.v = order.v;
  SDI_F->speedRef.w = order.w;
  SDI_F->speedRef.vmax = order.vmax;
  SDI_F->speedRef.wmax = order.wmax;
  SDI_F->speedRef.linAccelMax = order.linAccelMax;
  SDI_F->speedRef.angAccelMax = order.angAccelMax;
  /* Signal the reference update */
  SDI_F->speedRef.numRef = prevRflexNumRef == 0 ? 1 : 0;

  return EXEC;
}
#else
ACTIVITY_EVENT
rflexTrackConfigExec (GENPOS_POSTER_NAME *posterName, int *bilan) { return ETHER; }
#endif



 /*************/
 /* TRACK END */
 /*************/

/* TrackSpeedStop  -  codel END de TrackSpeedStart
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
TrackSpeedStop (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  rflexStopTracking();
  return ETHER;
}

#ifdef USE_CONTLAWS
/* rflexTrackConfigStop  -  codel END de rflexTrackStart
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rflexTrackConfigStop (GENPOS_POSTER_NAME *posterName, int *bilan)
{
  rflexStopTracking();

  /* set the previous speed to 0 */
  prevV = 0.0;
  prevW = 0.0;

  return ETHER;
}
#else
ACTIVITY_EVENT
rflexTrackConfigStop (GENPOS_POSTER_NAME *posterName, int *bilan)
{ return ETHER; }
#endif

