/**
 ** rflexRflexTaskCodels.c
 **
 ** Codels appeles par la tache rflexRflexTask
 **
 ** Auteur:
 ** Date:
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <stdio.h>
#include <fcntl.h>

#include "rflexConst.h"
#include "server/rflexHeader.h"
#include "h2timeLib.h"

#include <rFlex.h>
#include <rFlexSerial.h>
#include "rflexClient.h"
#include "rflexCodels.h"
#ifdef GYRO
#include "gyroLib.h"
#endif
#include "odo.h"
#include "rflexSonars.h"

#ifdef RFLEX_SIMUL
#include "rflexSimul.h"
#endif

/*
 * Variables globales
 */
struct reflex *reflex = NULL;
int rFlexFd = -1;

static POSTER_ID attitudePosterId;

/*------------------------------------------------------------------------
 *
 * initSensors  --  Codel d'initialisation (SDI/f, ...)
 *
 * Description:
 *
 * Retourne:    OK ou ERROR
 */

STATUS
initSensors(int *bilan)
{
	SDI_F->connect = RFLEX_NOT_CONNECTED;
	/* Default parameters for serial line  */
	/*strcpy(SDI_F->client.deviceName, RFLEX_DEFAULT_DEVICE_NAME);*/
	SDI_F->client.baudRate = RFLEX_DEFAULT_BAUD_RATE;

	/* all about gyro */
	SDI_F->gyro.initDone = FALSE;
	SDI_F->gyro.params.mode = RFLEX_GYRO_ON;
#ifdef GYRO
/* 	strcpy(SDI_F->gyro.params.port, RFLEX_GYRO_PORT); */
	SDI_F->gyro.gyroOn = FALSE;
#endif
	SDI_F->gyro.gyroToRobotOffset = 0;
	SDI_F->gyro.gyroTheta = 0;

	/* Default geomtric parameters */
	SDI_F->geomParams.linMToStep = RFLEX_LIN_M_TO_STEP;
	SDI_F->geomParams.angRadToStep = RFLEX_ANG_RAD_TO_STEP;
	SDI_F->geomParams.linStepToM = RFLEX_LIN_STEP_TO_M;
	SDI_F->geomParams.angStepToRad = RFLEX_ANG_STEP_TO_RAD;
	SDI_F->geomParams.sonarScale = RFLEX_SONAR_SCALE;
	SDI_F->geomParams.sonarOffset = RFLEX_SONAR_OFFSET;

	/* Start in AUTO mode */
	SDI_F->systemState.mode = AUTO_MODE;
	SDI_F->systemState.shock = 0;

	/* Initialize odomerty parameters */
	if (rFlexInitOdo(bilan) == ERROR) {
		return ERROR;
	}
	/* Initialize sonar sensors coordinates */
	if (rFlexComputeSensorsEchoes(bilan) == ERROR) {
		return ERROR;
	}

	/*give the old value of RFLEX_POSTER_REF_NO_UPDATE_MAX */
	SDI_F->WatchDogRef = RFLEX_DEFAULT_WATCHDOG_REF;

	return OK;
}


/*------------------------------------------------------------------------
 *
 * readSensors  --  codel de l'activite' permanente
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NO_MOTION
 *              S_rflex_EMERGENCY_STOP
 *              S_rflex_MANUAL
 *              S_rflex_LOW_LEVEL_BATTERY
 *              S_rflex_REF_NOT_UPDATED
 *
 * Retourne:    OK ou ERROR
 */

STATUS
readSensors(int *bilan)
{
	int nread;
	/* char buffer[256]; */

	if (rFlexFd < 0) {
		/* Not connected - nothing to do */
		return OK;
	}

	/* read data from rFlex */
	do {
		nread = reflex->read(reflex, 5);
		/* defaultHandlePort(reflex, -3, buffer, 0, 255, 255); */
	} while (nread != 0);


	/* print state change */
	rflexReadyForMotion(bilan);

	/* door switch on */
	if (SDI_F->systemState.shock) {
/* 	  printf ("rflex: shock ! (should call BrakeOnStart)\n"); */
	}

	if (SDI_F->systemState.brake) {
	  rflexStopTracking();
/* 	  printf ("rflex: set brake off\n");  */
	  BrakeOffStart(bilan);
	}

	/* Test brakes and battery */
	/* XXX TBD */

	/* Manual mode */
	if (SDI_F->systemState.mode == MANUAL_MODE) {
		JoystickDrive(reflex, bilan);
	}

	/* Send commands */
	if (Asserv(reflex, bilan) == ERROR) {
		return ERROR;
	}

	return OK;
}
/*------------------------------------------------------------------------
 * InitClient
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_CONNECTION_FAILED
 */

/* rFlexInitClient  -  codel EXEC de InitClient
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rFlexInitClient (RFLEX_CLIENT_STR *client, int *bilan)
{
#ifdef B21R
  int i,j;
#endif /* B21R */

	if (rFlexFd != -1) {
		/* Aleady connected - be gentle don't report an error */
		return ETHER;
	}
	/* Set up the call back functions for all messages */
	reflex = reflexSerialNew(client->deviceName, client->baudRate, &rFlexFd, NULL);
	if (!reflex)
	{
		*bilan = S_rflex_CONNECTION_FAILED;
		return ETHER;
	}
	reflex->handler = defaultHandlePort;
	reflex->system->lcdDumpRpl      = sysLcdDumpRpl;
	reflex->system->statusRpl       = sysStatusRpl;
	reflex->system->unknownRpl      = sysUnknownRpl;

	reflex->motion->getSystemRpl    = motAxisGetSystemRpl;
	reflex->motion->getModelRpl     = motAxisGetModelRpl;
	reflex->motion->getTargetRpl    = motAxisGetTargetRpl;
	reflex->motion->setDirRpl       = motAxisSetDirRpl;
	reflex->motion->setDefaultsRpl  = motSetDefaultsRpl;
	reflex->motion->brakeSetRpl     = motBrakeSetRpl;
	reflex->motion->brakeReleaseRpl = motBrakeReleaseRpl;
	reflex->motion->setSystemRptRpl = motSetSystemRptRpl;
	reflex->motion->unknownRpl      = motUnknownRpl;
#if ! defined RFLEX_SIMUL
	reflex->motion->systemRpt       = motAxisSystemRpt;
#else
	/* The report come from the simulation function */
	reflex->motion->systemRpt       = rflexSimulMotAxisSystemRpt;
#endif

	reflex->joystick->stateRpl      = jstkGetStateRpl;
	reflex->joystick->unknownRpl    = jstkUnknownRpl;

	reflex->sonar->updateRpl        = sonarGetUpdateRpl;
	reflex->sonar->rptRpl           = sonarRptRpl;
	reflex->sonar->unknownRpl       = sonarUnknownRpl;
	reflex->sonar->rpt              = sonarRpt;

#ifdef B21R
	SDI_F->dioTS = 0;
	SDI_F->dioCount = 0;
	SDI_F->dioOn = 10000;
/* 	SDI_F->dioState =0; */

	for(i=0;i<RFLEX_NB_DOORS;i++) {
	  for(j=0;j<RFLEX_NB_DOOR_SWITCHS;j++) {
	    SDI_F->doorSwitchs.tab[i][j] = 0;
	  }
	}

	for(i=0;i<RFLEX_NB_INFRA_REDS;i++) {
	  SDI_F->infraReds.tab[i] = 0;
	}

	reflex->dio->updateRpl          = dioUpdateRpl;
	reflex->dio->update             = dioUpdate;
	reflex->dio->rptRpl             = dioRptRpl;
	reflex->dio->rpt                = dioRpt;
	reflex->dio->unknownRpl         = dioUnknownRpl;

	/* Start io request */
	SDI_F->activeReq |= RFLEX_IO_REPORT;
	reflex->dio->rptReq(reflex, SDI_F->id, SDI_F->dioOn);
	INCREMENT(SDI_F->id);
#endif /* B21R */

	/* Start reporting axis data */
	reflex->motion->setSystemRptReq(reflex, SDI_F->id, 100000, 0x3);
	INCREMENT(SDI_F->id);

	/* Report initial system state */
	SDI_F->activeReq = RFLEX_SYSTEM_GET_STATE;
	reflex->system->statusReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);

	return ETHER;
}

/*------------------------------------------------------------------------
 * EndClient
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NOT_CONNECTED
 */

/* rFlexEndClient  -  codel EXEC de EndClient
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rFlexEndClient (int *bilan)
{
	/* ... add your code here ... */
	return ETHER;
}

/*------------------------------------------------------------------------
 * Stop
 *
 * Description:
 *
 * Bilans:      OK
 */

/* StopStart  -  codel START de Stop
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
StopStart (int *bilan)
{
	/* Reset motor commands to defaults */
	SDI_F->activeReq |= RFLEX_MOT_SET_DEFAULT;
	reflex->motion->setDefaultsReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);
	return EXEC;
}

/* Stop  -  codel EXEC de Stop
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
Stop (int *bilan)
{
	if (SDI_F->activeReq & RFLEX_MOT_SET_DEFAULT) {
		return EXEC;
	} else {
		return ETHER;
	}
}

/*------------------------------------------------------------------------
 * SonarOn
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NOT_CONNECTED
 */

/* SonarOnStart  -  codel START de SonarOn
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SonarOnStart(int *bilan)
{

	SDI_F->activeReq |= RFLEX_SONAR_REPORT;
	reflex->sonar->rptReq(reflex, SDI_F->id, 30000, 0, 0, 2);
	INCREMENT(SDI_F->id);
	return EXEC;
}

/* SonarOn  -  codel EXEC de SonarOn
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SonarOn(int *bilan)
{
	if (SDI_F->activeReq & RFLEX_SONAR_REPORT) {
		return EXEC;
	} else {
		return ETHER;
	}
}

/*------------------------------------------------------------------------
 * SonarOff
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NOT_CONNECTED
 */

/* SonarOffStart  -  codel START de SonarOff
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SonarOffStart(int *bilan)
{

	SDI_F->activeReq |= RFLEX_SONAR_REPORT;
	reflex->sonar->rptReq(reflex, SDI_F->id, 0, 0, 0, 0);
	INCREMENT(SDI_F->id);
	return EXEC;
}


/* SonarOff  -  codel EXEC de SonarOff
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SonarOff(int *bilan)
{

	if (SDI_F->activeReq & RFLEX_SONAR_REPORT) {
		return EXEC;
	} else {
		return ETHER;
	}
}

/*------------------------------------------------------------------------
 * BrakeOn
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NOT_CONNECTED
 */

/* BrakeOnStart  -  codel START de BrakeOn
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
BrakeOnStart(int *bilan)
{

	SDI_F->activeReq |= RFLEX_MOT_SET_BRAKES;
	reflex->motion->brakeSetReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);

	return EXEC;
}

/* BrakeOn  -  codel EXEC de BrakeOn
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
BrakeOn (int *bilan)
{

	if (SDI_F->activeReq & RFLEX_MOT_SET_BRAKES) {
		return EXEC;
	} else {
		return ETHER;
	}
}

/*------------------------------------------------------------------------
 * BrakeOff
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_NOT_CONNECTED
 */

/* BrakeOffStart  -  codel START de BrakeOff
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
BrakeOffStart(int *bilan)
{
        rflexStopTracking();

	SDI_F->activeReq |= RFLEX_MOT_REL_BRAKES;
	reflex->motion->brakeReleaseReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);

	return EXEC;
}

/* BrakeOff  -  codel EXEC de BrakeOff
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
BrakeOff (int *bilan)
{

	if (SDI_F->activeReq & RFLEX_MOT_REL_BRAKES) {
		return EXEC;
	} else {
		return ETHER;
	}
}


/*------------------------------------------------------------------------
 * GetJoystick
 *
 * Description:
 *
 * Bilans:      OK
 */

/* GetJoystickStart  -  codel START de GetJoystick
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GetJoystickStart (RFLEX_JOYSTICK_STR *joystickState, int *bilan)
{

	/* Manual mode */
	if (SDI_F->systemState.mode == MANUAL_MODE) {
		/* Joystick data is already up-to-date in the SDI */
		return ETHER;
	}
	/* Send joystick status request */
	SDI_F->activeReq |= RFLEX_JOY_GET_STATE;
	reflex->joystick->stateReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);
	return EXEC;
}

/* GetJoystick  -  codel EXEC de GetJoystick
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GetJoystick (RFLEX_JOYSTICK_STR *joystickState, int *bilan)
{

	/* Wait for joystick reply */
	if (SDI_F->activeReq & RFLEX_JOY_GET_STATE) {
		/* request still active */
		return EXEC;
	} else {
		return ETHER;
	}
}


/*------------------------------------------------------------------------
 * GetStatus
 *
 * Description:
 *
 * Bilans:      OK
 */

/* GetStatusStart  -  codel START de GetStatus
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GetStatusStart(RFLEX_SYSTEM_STATE_STR *systemState, int *bilan)
{

	SDI_F->activeReq |= RFLEX_SYSTEM_GET_STATE;
	reflex->system->statusReq(reflex, SDI_F->id);
	INCREMENT(SDI_F->id);
	return EXEC;
}

/* GetStatus  -  codel EXEC de GetStatus
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GetStatus(RFLEX_SYSTEM_STATE_STR *systemState, int *bilan)
{
	if (SDI_F->activeReq & RFLEX_SYSTEM_GET_STATE) {
		return EXEC;
	} else {
		return ETHER;
	}
}


/*----------------------------------------------------------------------*/

/*------------------------------------------------------------------------
 * MonitorBattery
 *
 * Description:
 *
 * Bilans:      OK
 */

/* rflexMonitorBatteryExec  -  codel EXEC de MonitorBattery
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT rflexMonitorBatteryExec (RFLEX_BATTERY_STATUS *statusToMonitor,
					int *bilan)
{
  switch(*statusToMonitor) {

  case RFLEX_BATTERY_HIGH:
    if (SDI_F->systemState.batteryStatus == RFLEX_BATTERY_HIGH)
      return ETHER;
    break;

  case RFLEX_BATTERY_MEDIUM:
    /* returns also if low */
    if (SDI_F->systemState.batteryStatus == RFLEX_BATTERY_LOW ||
	SDI_F->systemState.batteryStatus == RFLEX_BATTERY_MEDIUM)
      return ETHER;
    break;

  case RFLEX_BATTERY_LOW:
    if (SDI_F->systemState.batteryStatus == RFLEX_BATTERY_LOW)
      return ETHER;
    break;

  default:
    *bilan = S_rflex_BAD_CMD_DATA;
    break;
  }

  return EXEC;
}


/*------------------------------------------------------------------------
 * Gyro
 *
 * Description:
 *
 * Bilans:      OK
 */

/* rflexGyroExec  -  codel EXEC de Gyro
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT rflexGyroExec (RFLEX_GYRO_PARAMS *gyroParams, int *bilan)
{
  if (gyroParams->mode == SDI_F->gyro.currentMode)
    return ETHER;

  if (gyroParams->mode == RFLEX_GYRO_OFF) {
    SDI_F->gyro.currentMode = RFLEX_GYRO_OFF;
    return ETHER;
  }

#ifdef GYRO
  if (!SDI_F->gyro.initDone) {
    if (!(SDI_F->gyro.initDone = gyroInitAngle(gyroParams->port))) {
      *bilan = S_rflex_GYRO_ERROR;
      SDI_F->gyro.params.mode = RFLEX_GYRO_OFF;
      SDI_F->gyro.currentMode = RFLEX_GYRO_OFF;
      return ETHER;
    }
  }

  /* read gyro once */
  if (!(gyroReadCorrAngle(&SDI_F->gyro.gyroTheta))) {
    *bilan = S_rflex_GYRO_ERROR;
    SDI_F->gyro.params.mode = RFLEX_GYRO_OFF;
    SDI_F->gyro.currentMode = RFLEX_GYRO_OFF;
    SDI_F->gyro.initDone = FALSE;
    return ETHER;
  }

  /* reset gyro offset to match odo */
  SDI_F->gyro.gyroToRobotOffset = SDI_F->odo.robot.theta - SDI_F->gyro.gyroTheta;

  SDI_F->gyro.currentMode = SDI_F->gyro.params.mode;

#else
  printf ("rflexGyroExec: rflex module compiled without gyro support. Use -DGYRO\n");
  SDI_F->gyro.params.mode = RFLEX_GYRO_OFF;
  SDI_F->gyro.currentMode = RFLEX_GYRO_OFF;
  SDI_F->gyro.initDone = FALSE;
  *bilan = S_rflex_GYRO_ERROR;
#endif

  return ETHER;
}



/*------------------------------------------------------------------------
 * SetPosFromMEPoster
 *
 * Description:
 *
 * Bilans:      OK
 *              S_rflex_TRACKING_ON
 */

/* rflexSetPosFromMEPosterExec  -  codel EXEC de SetPosFromMEPoster
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT rflexSetPosFromMEPosterExec (RFLEX_STRING *posMEPosterName, int *bilan)
{
  POM_ME_POS pomMePos;
  RFLEX_CART_3DCONFIG setPosStr;
  POSTER_ID posterPosId;

  if (posterFind(posMEPosterName->name, &posterPosId) == ERROR) {
    *bilan = S_rflex_CANNOT_READ_POSTER;
    return ETHER;
  }

  if(rflexPOM_ME_POSPosterRead(posterPosId, &pomMePos) == ERROR) {
    *bilan = S_rflex_CANNOT_READ_POSTER;
    return ETHER;
  }

  setPosStr.x = pomMePos.main.euler.x;
  setPosStr.y = pomMePos.main.euler.y;
  setPosStr.z = pomMePos.main.euler.z;
  setPosStr.theta = pomMePos.main.euler.yaw;

  Recale (&setPosStr, bilan);

  /* vxx */
  SDI_F->odo.odoError.var[0] = pomMePos.main.var.vxx;
  /* vxy */
  SDI_F->odo.odoError.var[1] = pomMePos.main.var.vxy;
  /* vyy */
  SDI_F->odo.odoError.var[2] = pomMePos.main.var.vyy;
  /* vxt */
  SDI_F->odo.odoError.var[3] = pomMePos.main.var.vox;
  /* vyt */
  SDI_F->odo.odoError.var[4] = pomMePos.main.var.voy;
  /* vtt */
  SDI_F->odo.odoError.var[5] = pomMePos.main.var.voo;

  return ETHER;
}



/* rflexOdometryMethodExec  -  codel EXEC de OdometryMethod
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT rflexSetOdometryCommandExec (RFLEX_ODOMETRY_METHOD *params, int *bilan)
{
  if (params->mode == RFLEX_ODO_3D)
  {
      SDI_F->odometryMode = RFLEX_ODO_2D;

      if (rflexPOM_SENSOR_POSPosterFind(params->attitudePosterName, &attitudePosterId) == ERROR) {
	  *bilan = S_rflex_CANNOT_READ_POSTER;
	  return ETHER;
      }

      if (readAttitudeEstimation()) {
	  *bilan = S_rflex_CANNOT_READ_POSTER;
	  return ETHER;
      }

      SDI_F->odometryMode = RFLEX_ODO_3D;
  }

  return ETHER;
}



/*------------------------------------------------------------------------
 * Log
 *
 * Description:
 *
 * Reports:      OK
 *              S_rflex_CANNOT_OPEN_FILE
 *              S_rflex_CANNOT_WRITE_FILE
 */

static FILE *rflexLogFile;

/* rflexLogStart  -  codel START of Log
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rflexLogStart(GEN_STRING128 *fileName, int *report)
{
  if (!(rflexLogFile = fopen(fileName->name, "a"))) {
    printf ("rflex: cannot open odometer file %s\n", fileName->name);
    *report = S_rflex_CANNOT_OPEN_FILE;
    return ETHER;
  }

  printf ("rflexLogStart: log file %s found\n", fileName->name);
  return EXEC;
}

/* rflexLogExec  -  codel EXEC of Log
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rflexLogExec(GEN_STRING128 *fileName, int *report)
{
  H2TIMESPEC tv;

  /* get date */
  h2GetTimeSpec(&tv);

  /* save date */
  if (!fprintf(rflexLogFile, "%lf\n",
	       tv.tv_sec + tv.tv_nsec*1.0E-9)) {
    printf ("rflex: cannot log in %s\n", fileName->name);
    *report = S_rflex_CANNOT_WRITE_FILE;
    rflexLogInter(fileName, report);
    return ETHER;
  }

  /* save POM_ME_POS */
  fprintf(rflexLogFile, "date1 = ");
  print_int(rflexLogFile, &SDI_F->posPoster.date1, 1, 0, NULL, NULL);

  fprintf(rflexLogFile, "date2 = ");
  print_int(rflexLogFile,&SDI_F->posPoster.date2, 1, 0, NULL, NULL);

  fprintf(rflexLogFile, "main:\n");
  print_struct_POM_EULER (rflexLogFile, &SDI_F->posPoster.main.euler, 1, 0, NULL, NULL);


  fprintf(rflexLogFile, "VLocal:\n");
  print_struct_POM_EULER (rflexLogFile, &SDI_F->posPoster.VLocal.euler, 1, 0, NULL, NULL);


  return EXEC;
}

/* rflexLogInter  -  codel INTER of Log
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rflexLogInter(GEN_STRING128 *fileName, int *report)
{
  if (rflexLogFile) {
    fclose(rflexLogFile);
    rflexLogFile = NULL;
  }
  return ETHER;
}


/*
 * Read attitude estimation poster
 *   */
int
readAttitudeEstimation()
{
  POM_SENSOR_POS full_pos;
  if(rflexPOM_SENSOR_POSPosterRead(attitudePosterId, &full_pos) == ERROR) {
    // Fall back to ODO 2D
    printf("rflexOdometryMethod: Cannot read attitude estimation poster\n");
    return ERROR;
  }
  SDI_F->attitude = full_pos.sensorToMain.euler;
  return OK;
}

