/**
 ** Functions used to simulate rflex motion.
 **
 ** Instead of sending a motion request to the wheels (lib rflex-client),
 ** we send it to the simulation function and it computes the expected motion.
 ** Then, instead of receiving real value, we use the computed value.
 **
 ** Author: OLI
 ** Date: 24.04.2003
 **
 **/
#include <math.h>
#include <portLib.h>
#include <h2mathLib.h>
#include <rFlex.h>

#include "rflexConst.h"
#include "server/rflexHeader.h"
#include "rflexClient.h"
#include "odo.h"

#include "rflexSimul.h"

#ifdef RFLEX_SIMUL

static RFLEX_SIMUL_ODO reflexSimul; /* simulation of odo */

static int flagV = -1; /* useful for many things, as flaging */
static int flagW = -1; /* a SetPos action for instance */

/* Initialize the flags */
void rflexSimulInit () {
  flagV = -1;
  flagW = -1;
}

/* Compute the motion and set the result in reflexSimul structure */
STATUS
rflexSimul(double v,          /* speed reference to engine (lin or ang) */
	   int axis)          /* translation or rotation */
{
  double period = EXEC_TASK_PERIOD(RFLEX_RFLEXTASK_NUM);

#if 0
   static int loop = 0;


  if (loop == 1000 || loop ==2000)
    {
      loop =0;
      printf("v = %f, rflexSimul.x = %f", v, reflexSimul.x);
    }
  loop++;
#endif

  switch (axis) {
  case 0: /* translation Axis */
    if (flagV == -1) {
      /* first time */
      reflexSimul.x = SDI_F->odo.robot.xC;
      reflexSimul.y = SDI_F->odo.robot.yC;
    }
    else  {
      /* Update the internal structure */
      reflexSimul.x += v*period*cos(reflexSimul.theta);
      reflexSimul.y += v*period*sin(reflexSimul.theta);
    }
    flagV++;
    break;

  case 1: /* rotation axis */
    if (flagW == -1) {
      /* first time */
      reflexSimul.theta = SDI_F->odo.robot.theta;
    }
    else  {
      /* Update the internal structure */
      reflexSimul.theta += v*period;
    }
    flagW++;
    break;
  }

  return OK;

}

/* Fulfill odo data from rflexSimul structure */
int
rflexSimulMotAxisSystemRpt (struct reflex *reflex,
		  unsigned char id, int ts,
		  unsigned char axis,
		  int pos, int vel, int acc, int trq)
{
  static int haveV = 0, haveW = 0; /* flags used to compute odoProba at the right time */
  double direction;

  /* Update odo */
  switch (axis) {

  case 0: /* translation axis */

    if (flagV != -1) { /* no repositionning */
      /* Update global position */
      SDI_F->odo.robot.xC = reflexSimul.x;
      SDI_F->odo.robot.yC = reflexSimul.y;
    }
    else { /* the robot position has been changed manually */
      reflexSimul.x = SDI_F->odo.robot.xC;
      reflexSimul.y = SDI_F->odo.robot.yC;
    }
    flagV++;
    /* Update control point position */
    if (SDI_F->ref.backFlag == GENPOS_FORWARD_MOTION)
      direction = 1.0;
    else if (SDI_F->ref.backFlag == GENPOS_BACKWARD_MOTION)
      direction = -1.0;
    else direction = 0.0;
    SDI_F->odo.robot.x = SDI_F->odo.robot.xC + direction*
			 RFLEX_DIST_TO_CONTROLED_POINT*cos(SDI_F->odo.robot.theta);
    SDI_F->odo.robot.y = SDI_F->odo.robot.yC + direction*
			 RFLEX_DIST_TO_CONTROLED_POINT*sin(SDI_F->odo.robot.theta);
    haveV =1;
    break;

  case 1: /* rotation axis */
    if (flagW != -1) {     /* no repositionning */
      SDI_F->odo.robot.theta = reflexSimul.theta;
    }
    else {
      reflexSimul.theta = SDI_F->odo.robot.theta;
    }
    flagW++;
    haveW = 1;
    break;


  default:
    /* Bug */
    fprintf(stderr, "%s: unknown axis %d\n", __FUNCTION__, axis);
  }

  /* computes probabilistic odometry */
  if (haveV && haveW) {
    haveV = 0;
    haveW = 0;
    odoProba(&SDI_F->odo, RFLEX_AXIS_WIDTH, SDI_F->varParams.coeffLinAng,
		   EXEC_TASK_PERIOD(RFLEX_RFLEXTASK_NUM));
  }
  /* Store current timestamp */
  SDI_F->ts = ts;

  return(0);
}

#endif /* RFLEX_SIMUL */
