#ifndef _RFLEX_CONST_H
#define _RFLEX_CONST_H

/* ====================  ROBOT SPECIFIC'S VALUES ===================== */

#if defined(ATRV_JUNIOR)
#include "rflexConstATRV_JR.h"
#elif defined(ATRV)
#include "rflexConstATRV.h"
#elif defined(B21R)
#include "rflexConstB21R.h"
#else
#warning "No known platform defined, using default values"
#endif /* ATRV */

/* =============================================================== */
/*                   **** WARNING ****                             */
/*             these are ONLY DEFAULTS VALUES                      */
/*                                                                 */
/*              for ROBOT SPECIFIC VALUES,                         */
/*      see INCLUDED files at the BEGINNING OF THIS FILE :         */
/*                                                                 */
/*                  rflexConst<ROBOT>.h                            */
/* =============================================================== */

/* Repere dans lequel rflex calcule sa position */
#define RFLEX_FRAME	"Robot"

#define RFLEX_TICKS_PERIOD 4
#define RFLEX_SEC_PERIOD 0.040

/*#define RFLEX_GYRO_PORT "/dev/ttyR1"*/
/*#define RFLEX_DEFAULT_DEVICE_NAME "/dev/ttyR0"*/
#define RFLEX_DEFAULT_BAUD_RATE 115200

/* Now we want the control point to be the center */
/* #define RFLEX_DIST_TO_CONTROLED_POINT 0.5 */
/* #define RFLEX_DIST_CONTROLE_MIN 0.1 */
#define RFLEX_DIST_TO_CONTROLED_POINT 0.0
#define RFLEX_DIST_CONTROLE_MIN 0.0

/* The virtual wheel of Dala */
#define RFLEX_DIST_VIRT_WHEEL 1.0
/* It is used to limit its steering angle : 60 deg here */
#define RFLEX_MAX_STEERING_ANGLE 1.047


/* pour ne tuer personne... */
#ifndef RFLEX_VMAX_ROBOT
#define RFLEX_VMAX_ROBOT       0.25
#endif
#ifndef RFLEX_WMAX_ROBOT
#define RFLEX_WMAX_ROBOT       0.125 /* XX parametres pompes de h2 */
#endif
#ifndef RFLEX_AMAX_LIN_ROBOT
#define RFLEX_AMAX_LIN_ROBOT   1.
#endif
#ifndef RFLEX_AMAX_ANG_ROBOT
#define RFLEX_AMAX_ANG_ROBOT   0.5*(6. * M_PI / 2.)
#endif

#define RFLEX_LIN_TRQ_DEFAULT 30000
#define RFLEX_ANG_TRQ_DEFAULT 40000

/* Contants from mom ATRV2/Hardware/Edit Properties */
#define RFLEX_MAX_LIN_TRQ 30000
#define RFLEX_MAX_ANG_TRQ 35000

#define RFLEX_CLOCK 200 		/* rflex controller ticks/s */

/* Initial variances */
#define RFLEX_INIT_ERR_X 0.1
#define RFLEX_INIT_ERR_Y 0.1
#define RFLEX_INIT_ERR_THETA 0.03

#define RFLEX_INIT_VAR_COEFF_LIN             0.0
#define RFLEX_INIT_VAR_COEFF_ANG	     0.0
#define RFLEX_INIT_VAR_COEFF_LIN_ANG	     0.1   /* Le seul utilise' */

#define RFLEX_AXIS_WIDTH 0.68

/* valeur du test du Chi2 a 1 degres de liberte a 99% */
#define GAUSS_99 3.0 /* A peu prs ! */
#define SIGMA3_TO_VARIANCE(x) (SQR((x)/GAUSS_99))
#define VARIANCE_TO_SIGMA3(x)   (GAUSS_99*sqrt(x))


/* Default Maximum number of times the reference is allowed not to be updated 
   in tracking mode before generating an error */
/* maybe changed by SetWdogRef */
#define RFLEX_DEFAULT_WATCHDOG_REF 50

/* Conversion of sonar value */
/* Contants from mom ATRV2/Hardware/Edit Properties */
#define RFLEX_SONAR_SCALE   6.911E-4 
#define RFLEX_SONAR_OFFSET  0.01967

/* Lower acceptable battery level */
#define RFLEX_LOW_BATTERY_LEVEL	    21.2 /* volts */
#define RFLEX_MEDIUM_BATTERY_LEVEL  21.4 /* volts */
#define RFLEX_HIGH_BATTERY_LEVEL   23.5 /* volts */

/* Flags for ongoing requests */
#define RFLEX_SYSTEM_GET_STATE 0x0001
#define RFLEX_MOT_SET_BRAKES   0x0002
#define RFLEX_MOT_REL_BRAKES   0x0004
#define RFLEX_MOT_SET_DEFAULT  0x0008
#define RFLEX_MOT_SET_DIR      0x0010
#define RFLEX_JOY_GET_STATE    0x0020
#define RFLEX_SONAR_REPORT     0x0040


/*----------------------------------------------*/
/*------ Default parameters for Tracking -------*/
/*----------------------------------------------*/

/* Correction vitesse lineaire, gain proportionnel erreur longitudinale */
#define  RFLEX_KP_LONGIT                    1.0

/* Correction vitesse lineaire, gain integral erreur longitudinale */
#define  RFLEX_KI_LONGIT                    0.0 /* XXX 0.5 */

/* Correction vitesse angulaire, gain proportionnel, erreur transversale */
#define  RFLEX_KP_TRANSV                    1.0

/* Correction vitesse angulaire, gain integral, erreur transversale */
#define  RFLEX_KI_TRANSV                    0.0 /* XXX 0.5 */

/* Correction vitesse angulaire, gain proportionnel erreur angulaire */
#define  RFLEX_KP_ANG                       1.2

/* Correction vitesse angulaire, gain integral erreur angulaire */
#define  RFLEX_KI_ANG                       0

/* Max erreur longitud, pour prise en compte composante integrale vit. lin. */
#define  RFLEX_LONGIT_ERROR_MAX             0.1

/* Max. valeur de l'integrale d'erreurs longitudinales */
#define  RFLEX_LONGIT_MAX_ERR_SUM           0.5

/* Max erreur transverse, prise compte integrale */
#define  RFLEX_TRANSV_ERROR_MAX             0.1

/* Max. valeur de l'integrale d'erreurs transverses */
#define  RFLEX_TRANSV_MAX_ERR_SUM           0.5

/* Max erreur angulaire, pour prise en compte composante integrale vit. lin. */
#define  RFLEX_ANG_ERROR_MAX             0.15

/* Max. valeur de l'integrale d'erreurs angulaires */
#define  RFLEX_ANG_MAX_ERR_SUM           0.3


/*--------ECARTS AU DELA DESQUELS LE TRACKING EST INTERROMPU----  */
#define RFLEX_LONGIT_GAP_MAX           1
#define RFLEX_TRANSV_GAP_MAX           1
#define RFLEX_ANG_GAP_MAX              1

/* See rflexConstATRV.h or rflexConstB21R.h */
#ifndef RFLEX_ODO_LOG_NAME
#define RFLEX_ODO_LOG_NAME 0
#endif

#endif

